package com.good.ca.protocol.authentication;

import com.good.ca.protocol.ConnectionStatusCode;
import com.good.common.GDEntLogger;
import com.good.crypto.util.Base64String;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.conn.ssl.X509HostnameVerifier;
import org.apache.http.impl.client.DefaultHttpClient;

import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Objects;

/**
 * Abstract class for SSL connection. Contains common methods for Http Client creation.
 */

public abstract class AbstractSSLConnection {
    protected final String decryptPassword;
    protected final String keyStore;
    protected String certificate = null;
    private static GDEntLogger logger = GDEntLogger.getInstance(AbstractSSLConnection.class);

    protected AbstractSSLConnection(SSLConnectionDTO sslConnectionDTO) throws HttpAuthenticationException {
        this.decryptPassword = sslConnectionDTO.getDecryptPassword();
        this.keyStore = sslConnectionDTO.getStore();
    }

    public HttpClient getHttpClient() throws HttpAuthenticationException{
        logger.debug("getHttpClient start");
        DefaultHttpClient httpClient = null;
        try {
            KeyStore trustStore = getTrustStore();
            KeyStore keyStore = getKeyStore();
            X509Certificate cert = addX509CertificateToTrustStore(trustStore);
            TrustManager[] tm = getTrustManagers(trustStore, cert);
            KeyManager[] km = AuthenticationUtils.getKeyManagersFromKeyStore(keyStore, decryptPassword);
            httpClient = new DefaultHttpClient();
            SSLSocketFactory sslsf = getSSLSocketFactory(keyStore, trustStore, km, tm);
            Scheme https = new Scheme("https", 443, sslsf);
            httpClient.getConnectionManager().getSchemeRegistry().register(https);
        } catch (KeyManagementException | UnrecoverableKeyException | NoSuchAlgorithmException | KeyStoreException e) {
            throw new HttpAuthenticationException(e);
        }
        logger.debug("getHttpClient end");
        return httpClient;
    }

    protected KeyStore getTrustStoreFromGC() throws HttpAuthenticationException, KeyStoreException, NoSuchAlgorithmException {
        logger.debug("getTrustStore start");
        KeyStore trustStore = KeyStore.getInstance("jks");

        String filename = System.getProperty("java.home");
        if (filename != null) {
            logger.debug("getTrustStore adding GC trustStore to the connection");
            filename += "/lib/security/cacerts".replace('/', File.separatorChar);
            try (FileInputStream trustStoreStream = new FileInputStream(filename)) {
                trustStore.load(trustStoreStream, "changeit".toCharArray());
            } catch (CertificateException | IOException e) {
                throw new HttpAuthenticationException("Error loading keystore ",
                        e, ConnectionStatusCode.UNAUTHORIZED_ERROR);
            }

        } else {
            logger.debug("javahome is not set, GC trustore will not be set");
        }
        logger.debug("getTrustStore end");
        return trustStore;
    }

    private KeyStore getKeyStore() throws HttpAuthenticationException, UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException {
        logger.debug("getKeyStore start");
        KeyStore keyStore = null;
        if (Objects.nonNull(this.keyStore)) {
            keyStore = KeyStore.getInstance("pkcs12");
            try (InputStream keyStoreStream = getInputStream(this.keyStore)) {
                keyStore.load(keyStoreStream, decryptPassword.toCharArray());
                logger.debug("getKeyStore: keystore was loaded");
            } catch (CertificateException | IOException e) {
                throw new HttpAuthenticationException("Error loading keystore ",
                        e, ConnectionStatusCode.UNAUTHORIZED_ERROR);
            }
        }
        logger.debug("getKeyStore end");
        return keyStore;
    }

    private InputStream getInputStream(final String keyStore) {
        byte[] storeBytes = new Base64String(keyStore).toByteArray();
        return new ByteArrayInputStream(storeBytes);
    }

    private X509Certificate addX509CertificateToTrustStore(KeyStore trustStore) throws HttpAuthenticationException, KeyStoreException {
        logger.debug("addX509CertificateToTrustStore start");
        X509Certificate cert = null;
        if (certificate != null) {
            logger.debug("addX509CertificateToTrustStore certificate is not null");
            try (InputStream serverSslCertificateStream = getInputStream(certificate)) {
                CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
                cert = (X509Certificate) certFactory.generateCertificate(serverSslCertificateStream);
                cert.checkValidity();
                trustStore.setCertificateEntry("pkiconnectorcert", cert);
            } catch (IOException | CertificateException e) {
                throw new HttpAuthenticationException("Error loading keystore ",
                        e, ConnectionStatusCode.UNAUTHORIZED_ERROR);
            }
        }
        logger.debug("addX509CertificateToTrustStore end");
        return cert;
    }

    protected SSLSocketFactory getSSLSocketFactory(KeyManager[] km, TrustManager[] tm, X509HostnameVerifier hostnameVerifier)
            throws NoSuchAlgorithmException, KeyManagementException {
        SSLContext sslcontext = SSLContext.getInstance("TLS");
        sslcontext.init(km, tm, null);
        return new SSLSocketFactory(sslcontext, hostnameVerifier);
    }

    protected abstract TrustManager[] getTrustManagers(KeyStore trustStore, X509Certificate cert) throws KeyStoreException, NoSuchAlgorithmException;
    protected abstract SSLSocketFactory getSSLSocketFactory(KeyStore keyStore, KeyStore trustStore, KeyManager[] km, TrustManager[] tm)
            throws NoSuchAlgorithmException, KeyManagementException, UnrecoverableKeyException, KeyStoreException;
    protected abstract KeyStore getTrustStore() throws HttpAuthenticationException, KeyStoreException, NoSuchAlgorithmException;
}
