package com.good.ca.protocol.bean;

import com.good.crypto.util.Base64String;
import com.good.util.validation.annotation.NotEmpty;
import com.google.gson.JsonObject;

import java.util.Objects;

/**
 * Request for getUserKeyPair2 operation
 */
public class RequestCAKeyPair2 extends RequestCertificateAuthority {

    /**
     *{"initialCert", “renewCert”}
     */
    @NotEmpty
    private final RequestType mType;
    /**
     * User email addr or some other identifier.
     * Subject for the certificate is created by the issuer.
     */
    @NotEmpty
    private final String user;
    /**
     * OTP/password (for intialCert)
     */
    private final String authToken;
    /**
     * Only for initialCert. To assist sender to match response
     */
    private final String reqId;
    /**
     * Only for initialCert
     * GD device id. In some edge cases it is possible for GD container to change its deviceId.
     * This is here for informational and audit purposes.
     * It could also be used by issuer to track if two apps on the same device are making request for the certificate at the same time (or very close in time).
     */
    private final String deviceId;
    /**
     * Only for initialCert
     * Name of the device assigned by the user, which user can change at a later time.
     * This is here for informational and audit purposes.
     */
    private final String deviceName;
    /**
     * GC server sends this to the PKI Connector as received without doing any changes.
     */
    private final Base64String cmsSigned;

    private RequestCAKeyPair2(Builder builder) {
        this.mType = builder.mType;
        this.user = builder.user;
        this.authToken = builder.authToken;
        this.reqId = builder.reqId;
        this.deviceId = builder.deviceId;
        this.deviceName = builder.deviceName;
        this.cmsSigned = builder.cmsSigned;
        checkRequiredFields(this);
    }

    public static class Builder{
        private RequestType mType;
        private String user;
        private String authToken;
        private String reqId;
        private String deviceId;
        private String deviceName;
        private Base64String cmsSigned;

        public Builder (RequestType mType) {
            this.mType = mType;
        }

        public Builder user (String user) {
            this.user = user;
            return this;
        }

        public Builder authToken (String authToken) {
            this.authToken = authToken;
            return this;
        }

        public Builder reqId (String reqId) {
            this.reqId = reqId;
            return this;
        }

        public Builder deviceId (String deviceId) {
            this.deviceId = deviceId;
            return this;
        }

        public Builder deviceName (String deviceName) {
            this.deviceName = deviceName;
            return this;
        }

        public Builder cmsSigned (Base64String cmsSigned) {
            this.cmsSigned = cmsSigned;
            return this;
        }

        public RequestCAKeyPair2 build() {
            return new RequestCAKeyPair2(this);
        }
    }

    public RequestType getmType() {
        return mType;
    }

    public String getUser() {
        return user;
    }

    public String getAuthToken() {
        return authToken;
    }

    public String getReqId() {
        return reqId;
    }

    public String getDeviceId() {
        return deviceId;
    }

    public String getDeviceName() {
        return deviceName;
    }

    public Base64String getCmsSigned() {
        return cmsSigned;
    }

    @Override
    public String toString() {
        return "RequestCAKeyPair2{" +
                "mType=" + mType.getType() +
                ", user='" + user + '\'' +
                ", authToken='" + "***" + '\'' +
                ", reqId=" + reqId +
                ", deviceId='" + deviceId + '\'' +
                ", deviceName='" + deviceName + '\'' +
                ", cmsSigned=" + cmsSigned +
                '}';
    }

    @Override
    public JsonObject getMessage() {
        JsonObject message = new JsonObject();
        message.addProperty("mType", mType.getType());
        message.addProperty("user", user);
        message.addProperty("authToken", authToken);
        message.addProperty("reqId", reqId);
        message.addProperty("deviceId", deviceId);
        message.addProperty("deviceName", deviceName);
        if (Objects.nonNull(cmsSigned)) {
            message.addProperty("cmsSigned", cmsSigned.toString());
        }

        return message;
    }

    @Override
    public void setHttpType() {
        this.httpType = HttpType.POST;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        RequestCAKeyPair2 that = (RequestCAKeyPair2) o;
        return Objects.equals(mType, that.mType) &&
                Objects.equals(user, that.user) &&
                Objects.equals(authToken, that.authToken) &&
                Objects.equals(reqId, that.reqId) &&
                Objects.equals(deviceId, that.deviceId) &&
                Objects.equals(deviceName, that.deviceName) &&
                Objects.equals(cmsSigned, that.cmsSigned);
    }

    @Override
    public int hashCode() {
        return Objects.hash(mType, user, authToken, reqId, cmsSigned, deviceId, deviceName);
    }

}
