package com.good.ca.protocol.bean;

import com.good.util.validation.annotation.NotEmpty;
import com.good.crypto.util.Base64String;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

import java.util.Arrays;
import java.util.Objects;

/**
 * Request for certificateStatus operation
 */
public class RequestCertificateStatus extends RequestCertificateAuthority{
    /**
     * {received}
     * new public cert received by the GD runtime (either due to renewal or first time enrollment)
     */
    @NotEmpty
    private CertificateStatusType mType;
    /**
     * User email addr
     */
    @NotEmpty
    private String user;
    /**
     * x509
     */
    @NotEmpty
    private CertificateTypeEnum certType;
    /**
     * List of other certs present on the device ie. previous certs
     */
    @NotEmpty
    private Base64String[] otherCerts;
    /**
     * Certificate received
     */
    @NotEmpty
    private Base64String receivedCert;
    /**
     * GD device id. In some edge cases it is possible for GD container to change its deviceId.
     */
    private String deviceId;
    /**
     * Name of the device assigned by the user, which user can change at a later time.
     */
    private String deviceName;

    public enum CertificateStatusType {
        RECEIVED("received"),
        REMOVED("removed");
        private final String val;

        CertificateStatusType(final String val) {
            this.val = val;
        }

        public String getValue() {
            return this.val;
        }
    }
    public enum CertificateTypeEnum {

        PKCS12("pkcs12"),
        X509("x509");
        private final String val;

        CertificateTypeEnum(String val) {
            this.val = val;
        }
        public String getValue() {
            return this.val;
        }
    }

    private RequestCertificateStatus(Builder builder) {
        this.mType = builder.mType;
        this.user = builder.user;
        this.certType = builder.certType;
        this.otherCerts = builder.otherCerts;
        this.receivedCert = builder.receivedCert;
        this.deviceId = builder.deviceId;
        this.deviceName = builder.deviceName;
        checkRequiredFields(this);
    }
    public static class Builder{
        private CertificateStatusType mType;
        private String user;
        private CertificateTypeEnum certType;
        private Base64String[] otherCerts;
        private Base64String receivedCert;
        private String deviceId;
        private String deviceName;

        public Builder (CertificateStatusType mType) {
            this.mType = mType;
        }

        public Builder user (String user) {
            this.user = user;
            return this;
        }

        public Builder certType (CertificateTypeEnum certType) {
            this.certType = certType;
            return this;
        }

        public Builder otherCerts (Base64String[] otherCerts) {
            this.otherCerts = otherCerts;
            return this;
        }

        public Builder receivedCert (Base64String receivedCert) {
            this.receivedCert = receivedCert;
            return this;
        }

        public Builder deviceId (String deviceId) {
            this.deviceId = deviceId;
            return this;
        }

        public Builder deviceName (String deviceName) {
            this.deviceName = deviceName;
            return this;
        }

        public RequestCertificateStatus build() {
            return new RequestCertificateStatus(this);
        }
    }

    public CertificateStatusType getmType() {
        return mType;
    }

    public String getUser() {
        return user;
    }

    public CertificateTypeEnum getCertType() {
        return certType;
    }

    public Base64String[] getOtherCerts() {
        return otherCerts;
    }

    public Base64String getReceivedCert() {
        return receivedCert;
    }

    public String getDeviceId() {
        return deviceId;
    }

    public String getDeviceName() {
        return deviceName;
    }

    @Override
    public String toString() {
        return "RequestCertificateStatus{" +
                "mType=" + mType.getValue() +
                ", user='" + user + '\'' +
                ", certType='" + certType.getValue() + '\'' +
                ", otherCerts=" + Arrays.toString(otherCerts) +
                ", receivedCert=" + receivedCert +
                ", deviceId='" + deviceId + '\'' +
                ", deviceName='" + deviceName + '\'' +
                '}';
    }

    @Override
    public JsonObject getMessage() {
        JsonObject message = new JsonObject();
        message.addProperty("mType", mType.getValue());
        message.addProperty("user", user);
        message.addProperty("certType", certType.getValue());
        JsonArray otherCertsArray = new JsonArray();
        for (Base64String otherCert : otherCerts) {
            otherCertsArray.add(new JsonPrimitive(otherCert.toString()));
        }
        message.add("otherCerts", otherCertsArray);
        message.addProperty("receivedCert", receivedCert.toString());
        message.addProperty("deviceId", deviceId);
        message.addProperty("deviceName", deviceName);

        return message;
    }

    @Override
    public void setHttpType() {
        this.httpType = RequestCertificateAuthority.HttpType.POST;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        RequestCertificateStatus that = (RequestCertificateStatus) o;
        return Objects.equals(mType, that.mType) &&
                Objects.equals(user, that.user) &&
                Objects.equals(certType, that.certType) &&
                Objects.equals(otherCerts, that.otherCerts) &&
                Objects.equals(receivedCert, that.receivedCert) &&
                Objects.equals(deviceId, that.deviceId) &&
                Objects.equals(deviceName, that.deviceName);
    }

    @Override
    public int hashCode() {
        return Objects.hash(mType, user, certType, otherCerts, receivedCert, deviceId, deviceName);
    }

}
