package com.good.ca.protocol.bean;

import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

import com.good.crypto.util.Base64String;
import com.good.util.validation.annotation.NotEmpty;

import java.util.List;
import java.util.Objects;



/**
 * Request for notifyCertificateRemoved operation
 */
public class RequestNotifyCertificateRemoved extends RequestCertificateAuthority {
	/**
	 * User email address
	 */
	@NotEmpty
	private final String user;
	/**
	 * Certificates to be removed
	 */
	@NotEmpty
	private final List<Base64String> removedCerts; // Base64 encoded certificate
	
	/**
	 * Reason why this certificate was removed
	 */
	private final String reason;
	/**
	 * GD device id. In some edge cases it is possible for GD container to change its deviceId.
	 */
	private final String deviceId;
	/**
	 * Name of the device assigned by the user, which user can change at a later time.
	 */
	private final String deviceName;
	
	
	
	public RequestNotifyCertificateRemoved(String user, List<Base64String> removedCerts, String reason, String deviceId, String deviceName ) {
		this.user = user;
		this.removedCerts = removedCerts;
		this.reason = reason;
		this.deviceId = deviceId;
		this.deviceName = deviceName;
		checkRequiredFields(this);
	}

	private RequestNotifyCertificateRemoved(Builder builder) {
		this.user = builder.user;
		this.removedCerts = builder.removedCerts;
		this.reason = builder.reason;
		this.deviceId = builder.deviceId;
		this.deviceName = builder.deviceName;
		checkRequiredFields(this);
	}

	public static class Builder{
		private String user;
		private List<Base64String> removedCerts;
		private String reason;
		private String deviceId;
		private String deviceName;

		public Builder(String user) {
			this.user = user;
		}

		public Builder removedCerts(List<Base64String> removedCerts) {
			this.removedCerts = removedCerts;
			return this;
		}

		public Builder reason(String reason) {
			this.reason = reason;
			return this;
		}

		public Builder deviceId(String deviceId) {
			this.deviceId = deviceId;
			return this;
		}

		public Builder deviceName(String deviceName) {
			this.deviceName = deviceName;
			return this;
		}

		public RequestNotifyCertificateRemoved build() {
			return new RequestNotifyCertificateRemoved(this);
		}
	}

	public String getUser() {
		return user;
	}

	public List<Base64String> getRemovedCerts() {
		return removedCerts;
	}

	public String getReason() {
		return reason;
	}

	public String getDeviceId() {
		return deviceId;
	}

	public String getDeviceName() {
		return deviceName;
	}

	@Override
	public String toString() {
		return "RequestNotifyCertificateRemoved {" +
				"user='" + user + '\'' +
				", removedCerts='" + removedCerts + '\'' +
				", reason='" + reason + '\'' +
				", deviceId='" + deviceId + '\'' +
				", deviceName='" + deviceName + '\'' +
				'}';
	}

	@Override
	public JsonObject getMessage() {
		JsonObject message = new JsonObject();
		message.addProperty("user", user);
		JsonArray certsArray = new JsonArray();
		for(Base64String cert : removedCerts) /* Base 64 enocoded cert */
			certsArray.add(new JsonPrimitive(cert.toString()));
		message.add("removedCerts", certsArray);
		message.addProperty("reason", reason);
		message.addProperty("deviceId", deviceId);
		message.addProperty("deviceName", deviceName);

		return message;
	}

	@Override
	public void setHttpType() {
		this.httpType = RequestCertificateAuthority.HttpType.POST;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		RequestNotifyCertificateRemoved that = (RequestNotifyCertificateRemoved) o;
		return  Objects.equals(user, that.user) &&
				(removedCerts==null ? that.removedCerts==null : removedCerts.equals(that.removedCerts)) &&
				Objects.equals(reason, that.reason) &&
				Objects.equals(deviceId, that.deviceId) &&
				Objects.equals(deviceName, that.deviceName);
	}
	

	@Override
	public int hashCode() {
		return Objects.hash(user, removedCerts, reason, deviceId, deviceName);
	}
	
}
