package com.good.ca.protocol.service;

import com.good.ca.protocol.ConnectionStatusCode;
import com.good.ca.protocol.authentication.BasicAuthentication;
import com.good.ca.protocol.authentication.HttpAuthentication.AuthenticationType;
import com.good.ca.protocol.authentication.HttpAuthenticationException;
import com.good.ca.protocol.bean.*;
import com.good.ca.protocol.bean.RequestCertificateAuthority.HttpType;
import com.good.common.GDEntLogger;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicHeader;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.json.JSONException;

import java.io.IOException;

import static com.good.ca.protocol.service.ProtocolOperation.*;
import static java.nio.charset.StandardCharsets.UTF_8;

public class CertificateAuthorityServiceImpl implements CertificateAuthorityService {

    private static final GDEntLogger logger = GDEntLogger.getInstance(CertificateAuthorityServiceImpl.class);

    @Override
    public ResponseCAKeyPair requestKeyPair(String url, final RequestCAKeyPair request) throws CertificateAuthorityServiceException {
        ResponseCAKeyPair caKeyPair = null;
        try {
            logger.debug("Request KeyPair  for user: " + request.getUserEmail() + ", device Id: " + request.getDeviceId() +  ", device name: " + request.getDeviceName() + ", with auth: " + ((request.getAuthentication() != null)? request.getAuthentication().getType()+ ";" : "null;"));
            HttpResponse httpResponse = getHttpResponse(url + GET_KEYPAIR_OPERATION.getOperationEndpoint(), request);
            String httpEntityContent = getHttpEntityContent(httpResponse, url);
            if (StringUtils.isNotEmpty(httpEntityContent)) {
                caKeyPair = new ResponseCAKeyPair(httpEntityContent);
            } else{
                logger.error("KeyPair response is empty. Null RequestCAKeyPair will be returned");
            }
        } catch (Exception e) {
            throw new CertificateAuthorityServiceException(e);
        }
        return caKeyPair;
    }

    @Override
    public ResponseCARevoke requestRevoke(final String url, final RequestCARevoke revokeRequest) throws CertificateAuthorityServiceException {
        ResponseCARevoke revoke = null;
        try {
            HttpResponse httpResponse = getHttpResponse(url, revokeRequest);
            String httpEntityContent = getHttpEntityContent(httpResponse, url);
            if (StringUtils.isNotEmpty(httpEntityContent))
                revoke = new ResponseCARevoke(httpEntityContent);
        } catch (Exception e) {
            throw new CertificateAuthorityServiceException(e);
        }
        return revoke;
    }

    @Override
    public ResponseCAGetInfo getInfo(String url, final RequestCAGetInfo request) throws CertificateAuthorityServiceException, IOException, HttpAuthenticationException {
        ResponseCAGetInfo responseCAGetInfo = null;
        try {
            HttpResponse httpResponse = getHttpResponse(url + GET_INFO_OPERATION.getOperationEndpoint(), request);
            logger.debug("Following status code was received in response: " + httpResponse.getStatusLine().getStatusCode());
            if (httpResponse.getStatusLine().getStatusCode() == 401) {
                throw new HttpAuthenticationException("Unauthorized", ConnectionStatusCode.UNAUTHORIZED_ERROR);
            } else if (httpResponse.getStatusLine().getStatusCode() != 200) {
                throw new HttpAuthenticationException("Unexpected Status Code", ConnectionStatusCode.CONNECTION_ERROR);
            }
            HttpEntity entity = httpResponse.getEntity();
            if (entity == null) {
                throw new CertificateAuthorityServiceException("HttpEntity is null. url: " + url);
            }
            String httpEntityContent = EntityUtils.toString(entity);
            logger.debug("Response httpEntityContent = " + httpEntityContent);
            if (StringUtils.isNotEmpty(httpEntityContent)) {
                responseCAGetInfo = new ResponseCAGetInfo(httpEntityContent);
            }
            return responseCAGetInfo;
        } catch (JSONException e) {
            throw new CertificateAuthorityServiceException(e);
        }
    }
    
    @Override
    public ResponseSignCertificate signCertificate(String url, RequestSignCertificate request) throws CertificateAuthorityServiceException {
        ResponseSignCertificate response = null;
        try {
            logger.debug("RequestSignCertificate received: " + request);
            HttpResponse httpResponse = getHttpResponse(url + SIGN_CERTIFICATE_OPERATION.getOperationEndpoint(), request);
            String httpEntityContent = getHttpEntityContent(httpResponse, url);
            if (StringUtils.isNotEmpty(httpEntityContent)) {
                response = new ResponseSignCertificate(httpEntityContent);
                logger.debug("ResponseSignCertificate received: " + response);
            } else {
                logger.error("Response is empty. Null ResponseSignCertificate will be returned");
            }
        }  catch (Exception e) {
            throw new CertificateAuthorityServiceException (e);
        }
        return response;
    }

    @Override
    public ResponseCAKeyPair2 getUserKeyPair2(String url, RequestCAKeyPair2 request) throws CertificateAuthorityServiceException {
        ResponseCAKeyPair2 response = null;
        try {
            logger.debug("RequestCAKeyPair2 received: " + request);
            HttpResponse httpResponse = getHttpResponse(url + GET_KEYPAIR_2_OPERATION.getOperationEndpoint(), request);
            String httpEntityContent = getHttpEntityContent(httpResponse, url);
            if (StringUtils.isNotEmpty(httpEntityContent)) {
                response = new ResponseCAKeyPair2(httpEntityContent);
                logger.debug("ResponseCAKeyPair2 received: " + response);
            } else {
                logger.error("KeyPair response is empty. Null ResponseCAKeyPair2 will be returned");
            }
        }  catch (Exception e) {
            throw new CertificateAuthorityServiceException (e);
        }
        return response;
    }

    @Override
    public ResponseNotifyCertificateReceived notifyCertificateReceived(final String url, final RequestNotifyCertificateReceived request)
            throws CertificateAuthorityServiceException {
        ResponseNotifyCertificateReceived response = null;
        try {
            logger.debug("RequestNotifyCertificateReceived received: " + request);
            HttpResponse httpResponse = getHttpResponse(url + NOTIFY_CERTIFICATE_RECEIVED.getOperationEndpoint(), request);
            String httpEntityContent = getHttpEntityContent(httpResponse, url);
            if (StringUtils.isNotEmpty(httpEntityContent)) {
                response = new ResponseNotifyCertificateReceived(httpEntityContent);
                logger.debug("ResponseNotifyCertificateReceived received: " + response);
            } else {
                logger.error("Response is empty. Null ResponseNotifyCertificateReceived will be returned");
            }
        }  catch (Exception e) {
            throw new CertificateAuthorityServiceException(e);
        }
        return response;
    }

    @Override
    public ResponseNotifyCertificateRemoved notifyCertificateRemoved(final String url, final RequestNotifyCertificateRemoved request)
            throws CertificateAuthorityServiceException {
        ResponseNotifyCertificateRemoved response = null;
        try {
            logger.debug("RequestNotifyCertificateRemoved received: " + request);
            HttpResponse httpResponse = getHttpResponse(url + NOTIFY_CERTIFICATE_REMOVED.getOperationEndpoint(), request);
            String httpEntityContent = getHttpEntityContent(httpResponse, url);
            if (StringUtils.isNotEmpty(httpEntityContent)) {
                response = new ResponseNotifyCertificateRemoved(httpEntityContent);
                logger.debug("ResponseNotifyCertificateRemoved received: " + response);
            } else {
                logger.error("Response is empty. Null ResponseNotifyCertificateRemoved will be returned");
            }
        }  catch (Exception e) {
            throw new CertificateAuthorityServiceException(e);
        }
        return response;
    }

    HttpResponse getHttpResponse(final String url, final IRequestCertificateAuthority request) throws CertificateAuthorityServiceException, IOException, HttpAuthenticationException {
        StringEntity input;
        HttpResponse httpResponse = null;

        if (request.getHttpType() == HttpType.POST) {
            try {
                input = new StringEntity(request.getMessage().toString(), HTTP.UTF_8);
                input.setChunked(false);
            } catch (Exception e) {
                throw new CertificateAuthorityServiceException(e);
            }
            HttpPost postRequest = new HttpPost(url);
            HttpParams params = new BasicHttpParams();
            //WE DONT SUPPORT CHUNKED RESPONSE FROM THE SERVER SO ACT A HTTP CLIENT SUPPORTING HTTTP 1.0 PROTOCOL
            //FORCE ADDING AUTHORIZATION HEADER FOR SANITY
            if (request.getAuthentication().getType() == AuthenticationType.BASIC_AUTH) {
                BasicAuthentication bda = (BasicAuthentication) request.getAuthentication();
                String authorization = "Basic " + new String(Base64.encodeBase64((bda.getUserName() + ":" + bda.getPassWord()).getBytes(UTF_8)), UTF_8);
                postRequest.setHeader(new BasicHeader("Authorization", authorization));
            }

            params.setParameter("http.protocol.version", HttpVersion.HTTP_1_0);
            postRequest.setParams(params);
            input.setContentType("application/json" + HTTP.CHARSET_PARAM + HTTP.UTF_8);
            postRequest.setEntity(input);
            logger.debug("Sending POST request. URL = " + url + ". Body = " + postRequest.getEntity());
            httpResponse = request.getAuthentication().getClient().execute(postRequest);

        } else if (request.getHttpType() == HttpType.GET) {
            HttpGet getRequest = new HttpGet(url);
            if (request.getAuthentication().getType() == AuthenticationType.BASIC_AUTH) {
                BasicAuthentication bda = (BasicAuthentication) request.getAuthentication();
                String authorization = "Basic " + new String(Base64.encodeBase64((bda.getUserName() + ":" + bda.getPassWord()).getBytes(UTF_8)), UTF_8);
                getRequest.setHeader(new BasicHeader("Authorization", authorization));
            }
            logger.debug("Sending GET request. URL = " + url);
            httpResponse = request.getAuthentication().getClient().execute(getRequest);
        }
        if (httpResponse == null) {
            throw new CertificateAuthorityServiceException("httpResponse is null. url: " + url);
        }
        return httpResponse;
    }

    private String getHttpEntityContent(HttpResponse httpResponse, String url) throws IOException, CertificateAuthorityServiceException {
        if (httpResponse != null && url != null) {
            HttpEntity entity = httpResponse.getEntity();
            if (entity == null) {
                throw new CertificateAuthorityServiceException("HttpEntity is null. url: " + url);
            }
            return EntityUtils.toString(entity);
        }
        return null;
    }
}
