package com.good.ca.protocol.service;

import com.good.ca.protocol.bean.PkiConnectorCapabilities;
import com.good.common.GDEntLogger;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * Endpoints specified in GD User Management Protocol. These operations must be supported by external CA servers
 *
 * @author Ivan Bilyi
 * @author Stanislav Kyfenko
 */
public enum ProtocolOperation {
    GET_KEYPAIR_OPERATION("getUserKeyPair"),
    GET_INFO_OPERATION("getInfo"),
    GET_KEYPAIR_2_OPERATION("getUserKeyPair2"),
    SIGN_CERTIFICATE_OPERATION("signCertificate"),
    CERTIFICATE_STATUS_OPERATION("certificateStatus"), // Deprecated, to be removed..
    NOTIFY_CERTIFICATE_RECEIVED("notifyCertificateReceived"),
    NOTIFY_CERTIFICATE_REMOVED("notifyCertificateRemoved");

    private static final GDEntLogger logger = GDEntLogger.getInstance(ProtocolOperation.class);

    private final String operationName;

    private static final Map<ProtocolOperation, PkiConnectorCapabilities[]> connectorCapabilitiesMapping;

    static {
        connectorCapabilitiesMapping = new HashMap<>(8);
        connectorCapabilitiesMapping.put(GET_KEYPAIR_OPERATION, new PkiConnectorCapabilities[]{PkiConnectorCapabilities.GET_P12});
        connectorCapabilitiesMapping.put(GET_KEYPAIR_2_OPERATION, new PkiConnectorCapabilities[]{PkiConnectorCapabilities.GET_P12, PkiConnectorCapabilities.RENEW_CERT});
        connectorCapabilitiesMapping.put(SIGN_CERTIFICATE_OPERATION, new PkiConnectorCapabilities[]{PkiConnectorCapabilities.RENEW_CERT});
        connectorCapabilitiesMapping.put(NOTIFY_CERTIFICATE_RECEIVED, new PkiConnectorCapabilities[]{PkiConnectorCapabilities.NOTIFY_CERTIFICATE_RECEIVED});
        connectorCapabilitiesMapping.put(NOTIFY_CERTIFICATE_REMOVED, new PkiConnectorCapabilities[]{PkiConnectorCapabilities.NOTIFY_CERTIFICATE_REMOVED});
    }

    ProtocolOperation(String operationName) {
        this.operationName = operationName;
    }

    public String getOperationEndpoint() {
        return "/pki?operation=" + operationName;
    }

    public String getOperationName() {
        return operationName;
    }

    public static PkiConnectorCapabilities[] convertToPKIConnectorCapabilities(ProtocolOperation operation) {
        return connectorCapabilitiesMapping.get(operation);
    }

    public static ProtocolOperation getByType(String type) {
        for (ProtocolOperation enumType : ProtocolOperation.values()) {
            if (enumType.getOperationName().equals(type)) {
                return enumType;
            }
        }
        return null;
    }

    //format for sending over CMP and to UI
    public static String convertConnectorCapabilitiesForCMPAndUI(List<String> supportedOperations) {
        logger.debug("convert connector capabilities. input " + supportedOperations);
        if (CollectionUtils.isEmpty(supportedOperations)) {
            return "";
        }
        String result = supportedOperations.stream()
                .map(ProtocolOperation::getByType)
                .map(ProtocolOperation::convertToPKIConnectorCapabilities)
                .filter(Objects::nonNull)
                .flatMap(Arrays::stream)
                .map(PkiConnectorCapabilities::getValue)
                .distinct()
                .collect(Collectors.joining(", "));
        logger.debug("convert connector capabilities. output " + result);
        return result;
    }

    //format for sending over CMP and to UI
    public static String convertConnectorCapabilitiesForCMPAndUI(String supportedOperations) {
        logger.debug("convert connector capabilities. input " + supportedOperations);
        if (StringUtils.isEmpty(supportedOperations)) {
            return "";
        }
        return convertConnectorCapabilitiesForCMPAndUI(Arrays.asList(supportedOperations.split(", ")));
    }

    // format for storing in DB
    public static String convertConnectorCapabilitiesForDB(List<String> supportedOperations) {
        logger.debug("convert connector capabilities to db. input " + supportedOperations);
        if (CollectionUtils.isEmpty(supportedOperations)) {
            return "";
        }
        // check whether protocol operation exists, filter nullable and join by comma
        String result = supportedOperations.stream()
                .map(ProtocolOperation::getByType)
                .filter(Objects::nonNull)
                .map(ProtocolOperation::getOperationName)
                .collect(Collectors.joining(", "));
        logger.debug("convert connector capabilities to db. output " + result);
        return result;
    }
}
