package com.good.ca.protocol.service;

import com.good.ca.protocol.ConnectionStatusCode;
import com.good.ca.protocol.authentication.BasicAuthentication;
import com.good.ca.protocol.authentication.HttpAuthenticationException;
import com.good.ca.protocol.bean.*;
import com.good.ca.protocol.model.InfoResponseData;
import com.good.ca.protocol.model.JSONConvertible;
import com.good.ca.protocol.model.KeyPairResponseData;
import com.good.ca.protocol.model.RevokeResponseData;
import org.junit.Assert;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.entity.StringEntity;
import org.json.JSONException;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import static org.mockito.Mockito.*;

/**
 *  Class which test gd-ca-protocol module, service CertificateAuthorityServiceImpl
 *
 * Created by omoniat on 17.03.2016.
 */

public class CertificateAuthorityServiceTest {


    public static final String DEFAULT_USER = "user";
    public static final String DEFAULT_PASSWORD = "password";
    public static final String REQ_ID = "1";
    public static final String BASE_URL = "url";
    public static final String DEFAULT_OTP = "otp";
    public static final String DEFAULT_USER_EMAIL = "mail@mail.com";
    public static final String DEFAULT_HEADER = "header";
    public static final String DEFAULT_SIGNATURE = "signature";
    public static final RequestType INITIAL_REQUEST = RequestType.INITIAL_CERT;
    public static final RequestType RENEW_REQUEST = RequestType.RENEW_CERT;
    public static final String DEFAULT_DEVICE_ID = "123jblj1b23bb123jbljb";
    public static final String DEFAULT_DEVICE_NAME = "My Iphone 6S";
    public static final InfoResponseData INFO_OPERATIONS = new InfoResponseData(new String[]{"getInfo", "getUserKeyPair"});
    public static final List<String> INFO_OPERATIONS_LIST = Arrays.asList("getInfo","getUserKeyPair");
    public static final String GET_KEYPAIR_OPERATION = "/pki?operation=getUserKeyPair";
    public static final String GET_INFO_OPERATION = "/pki?operation=getInfo";
    public static final RequestCARevoke REVOKE_REQUEST = new RequestCARevoke();

    static {
        REVOKE_REQUEST.setPublicCert("publicCert");
        REVOKE_REQUEST.setReason("reason");
    }

    private enum KeyPairResponseEnum {

        GOOD_REQUEST(new KeyPairResponseData(REQ_ID,"success", "", "payload", ResponseCAKeyPair.PayLoadType.PKCS12, "password")),
        GOOD_REQUEST_NO_PASSWORD(new KeyPairResponseData(REQ_ID, "success", "", "payload", ResponseCAKeyPair.PayLoadType.PKCS12, "")),
        BAD_REQUEST(new KeyPairResponseData(REQ_ID, "failure", "badRequest", "", ResponseCAKeyPair.PayLoadType.PKCS12, ""));

        private final KeyPairResponseData responseData;

        KeyPairResponseEnum(KeyPairResponseData responseData) {
            this.responseData = responseData;
        }

        public KeyPairResponseData getResponseData() {
            return responseData;
        }

    }

    private static Map<RequestCAKeyPair, KeyPairResponseData> requestResponseKeyPairMap;

    @BeforeClass
    public static void prepareKeyPairMockMap() throws CertificateAuthorityServiceException {
        requestResponseKeyPairMap = new HashMap<>();
        RequestCAKeyPair correctUserKeyPairInitial = getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, INITIAL_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME);
        requestResponseKeyPairMap.put(correctUserKeyPairInitial, KeyPairResponseEnum.GOOD_REQUEST.getResponseData());
        RequestCAKeyPair correctUserKeyPairInitialWithouToken = getKeyPairRequest(null, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, INITIAL_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME);
        requestResponseKeyPairMap.put(correctUserKeyPairInitialWithouToken, KeyPairResponseEnum.GOOD_REQUEST_NO_PASSWORD.getResponseData());
        RequestCAKeyPair correctUserKeyPairRenew = getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, RENEW_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME);
        requestResponseKeyPairMap.put(correctUserKeyPairRenew, KeyPairResponseEnum.GOOD_REQUEST.getResponseData());
        RequestCAKeyPair emptyKeyPair =  getKeyPairRequest(null, null, null, null, null, null, null);
        requestResponseKeyPairMap.put(emptyKeyPair, KeyPairResponseEnum.BAD_REQUEST.getResponseData());
        RequestCAKeyPair emptyUserKeyPair = getKeyPairRequest(DEFAULT_OTP, null, DEFAULT_HEADER, DEFAULT_SIGNATURE, INITIAL_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME);
        requestResponseKeyPairMap.put(emptyUserKeyPair, KeyPairResponseEnum.BAD_REQUEST.getResponseData());
        RequestCAKeyPair emptyTypeKeyPair = getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, null, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME);
        requestResponseKeyPairMap.put(emptyTypeKeyPair, KeyPairResponseEnum.BAD_REQUEST.getResponseData());
    }

    @Rule
    public ExpectedException expectedException = ExpectedException.none();

    @Test
    public void testCorrectInitialRequest() throws CertificateAuthorityServiceException, HttpAuthenticationException, IOException, JSONException {
        testKeyPairRequest(getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, INITIAL_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME));
    }

    @Test
    public void testCorrectInitialRequestWithoutAuthToken() throws CertificateAuthorityServiceException, HttpAuthenticationException, IOException, JSONException {
        testKeyPairRequest(getKeyPairRequest(null, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, INITIAL_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME));

    }

    @Test
    public void testCorrectRenewRequest() throws CertificateAuthorityServiceException, HttpAuthenticationException, IOException, JSONException {
        testKeyPairRequest(getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, RENEW_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME));
    }

    @Test
    public void testEmptyRequest() throws CertificateAuthorityServiceException, JSONException, HttpAuthenticationException, IOException {
        testKeyPairRequest(getKeyPairRequest(null, null, null, null, null, null, null));
    }

    @Test
    public void testEmptyUserRequest() throws CertificateAuthorityServiceException, JSONException, HttpAuthenticationException, IOException {
        testKeyPairRequest(getKeyPairRequest(DEFAULT_OTP, null, DEFAULT_HEADER, DEFAULT_SIGNATURE, INITIAL_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME));
    }

    @Test
    public void testEmptyTypeRequest() throws CertificateAuthorityServiceException, JSONException, HttpAuthenticationException, IOException {
        testKeyPairRequest(getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, null, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME));
    }

    @Test
    public void testConnectionErrorGetInfoRequest() throws CertificateAuthorityServiceException, JSONException, HttpAuthenticationException, IOException {
        expectedException.expect(HttpAuthenticationException.class);
        expectedException.expectMessage("Unexpected Status Code");
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = spy(new CertificateAuthorityServiceImpl());
        RequestCAGetInfo getInfoRequest = new RequestCAGetInfo();
        HttpResponse mockHttpResponse = getMockInfoHttpResponse(ConnectionStatusCode.CONNECTION_ERROR);
        doReturn(mockHttpResponse).when(certificateAuthorityServiceSpy).getHttpResponse(BASE_URL + GET_INFO_OPERATION, getInfoRequest);
        certificateAuthorityServiceSpy.getInfo(BASE_URL, getInfoRequest);
    }

    @Test
    public void testUnauthorizedGetInfoRequest() throws CertificateAuthorityServiceException, JSONException, HttpAuthenticationException, IOException {
        expectedException.expect(HttpAuthenticationException.class);
        expectedException.expectMessage("Unauthorized");
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = spy(new CertificateAuthorityServiceImpl());
        RequestCAGetInfo getInfoRequest = new RequestCAGetInfo();
        HttpResponse mockHttpResponse = getMockInfoHttpResponse(ConnectionStatusCode.UNAUTHORIZED_ERROR);
        doReturn(mockHttpResponse).when(certificateAuthorityServiceSpy).getHttpResponse(BASE_URL + GET_INFO_OPERATION, getInfoRequest);
        certificateAuthorityServiceSpy.getInfo(BASE_URL, getInfoRequest);
    }

    @Test
    public void testSuccessGetInfoRequest() throws CertificateAuthorityServiceException, JSONException, HttpAuthenticationException, IOException {
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = spy(new CertificateAuthorityServiceImpl());
        RequestCAGetInfo getInfoRequest = new RequestCAGetInfo();
        HttpResponse mockHttpResponse = getMockInfoHttpResponse(ConnectionStatusCode.SUCCESS);
        doReturn(mockHttpResponse).when(certificateAuthorityServiceSpy).getHttpResponse(BASE_URL + GET_INFO_OPERATION, getInfoRequest);
        ResponseCAGetInfo infoResponse = certificateAuthorityServiceSpy.getInfo(BASE_URL, getInfoRequest);
        Assert.assertEquals("Expect operations: " + INFO_OPERATIONS_LIST, INFO_OPERATIONS_LIST, infoResponse.getOperations());
    }

    @Test
    public void testRevokeRequest() throws IOException, HttpAuthenticationException, CertificateAuthorityServiceException, JSONException {
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = prepareRevokeRequestMockService(REVOKE_REQUEST, false);
        ResponseCARevoke revokeResponse = certificateAuthorityServiceSpy.requestRevoke(BASE_URL, REVOKE_REQUEST);
        Assert.assertNotNull("Status could not be empty", revokeResponse.getStatus());
    }

    @Test(expected = CertificateAuthorityServiceException.class)
    public void testEmptyKeyPairResponse() throws HttpAuthenticationException, IOException, CertificateAuthorityServiceException {
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = spy(new CertificateAuthorityServiceImpl());
        RequestCAKeyPair keyPairRequest = getKeyPairRequest(DEFAULT_OTP, DEFAULT_USER_EMAIL, DEFAULT_HEADER, DEFAULT_SIGNATURE, RENEW_REQUEST, DEFAULT_DEVICE_ID, DEFAULT_DEVICE_NAME);
        HttpResponse mockHttpResponse = mock(HttpResponse.class);
        when(mockHttpResponse.getEntity()).thenReturn(null);
        doReturn(mockHttpResponse).when(certificateAuthorityServiceSpy).getHttpResponse(BASE_URL + GET_KEYPAIR_OPERATION, keyPairRequest);
        certificateAuthorityServiceSpy.requestKeyPair(BASE_URL, keyPairRequest);
    }

    @Test(expected = CertificateAuthorityServiceException.class)
    public void testEmptyRevokeResponse() throws HttpAuthenticationException, IOException, CertificateAuthorityServiceException {
        System.out.println(REVOKE_REQUEST);
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = prepareRevokeRequestMockService(REVOKE_REQUEST, true);
        certificateAuthorityServiceSpy.requestRevoke(BASE_URL, REVOKE_REQUEST);
    }

    public void testKeyPairRequest(RequestCAKeyPair request) throws CertificateAuthorityServiceException, HttpAuthenticationException, IOException, JSONException {
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = spy(new CertificateAuthorityServiceImpl());
        KeyPairResponseData mockResponseData = requestResponseKeyPairMap.get(request);
        HttpResponse mockHttpResponse = getMockHttpResponse(mockResponseData, false);
        doReturn(mockHttpResponse).when(certificateAuthorityServiceSpy).getHttpResponse(BASE_URL + GET_KEYPAIR_OPERATION, request);
        ResponseCAKeyPair responseKeyPair = certificateAuthorityServiceSpy.requestKeyPair(BASE_URL, request);
        checkKeyPairResponse(responseKeyPair, mockResponseData, request);
    }

    private void checkKeyPairResponse(ResponseCAKeyPair actualResponse, KeyPairResponseData mockResponseData, RequestCAKeyPair requestKeyPair) throws JSONException {
        Assert.assertNotNull("Response is null", actualResponse);
        Assert.assertNotNull("Req id is null", actualResponse.getReqId());
        Assert.assertEquals("Status fields is not equal", mockResponseData.getStatus(), actualResponse.getStatus());
        Assert.assertEquals("Failure info is not equal", mockResponseData.getFailureInfo(), actualResponse.getFailureInfo());
        Assert.assertEquals("PayLoad type fields is not equal", mockResponseData.getPayLoadType(), actualResponse.getPayLoadType());
        Assert.assertEquals("PayLoad fields is not equal", mockResponseData.getPayload(), actualResponse.getPayLoad());
        if(requestKeyPair.getOtp() != null){
            Assert.assertEquals("Password fields is not equal", mockResponseData.getPassword(), actualResponse.getPassword());
        } else{
            Assert.assertTrue("Password must be empty", actualResponse.getPassword().isEmpty());
        }
    }

    private static RequestCAKeyPair getKeyPairRequest(String otp, String userEmail, String header, String signature, RequestType requestType, String deviceId, String deviceName){
        RequestCAKeyPair.Builder builder = new RequestCAKeyPair.Builder(requestType);
        builder.otp(otp);
        builder.userEmail(userEmail);
        builder.header(header);
        builder.signature(signature);
        builder.deviceId(deviceId);
        builder.deviceName(deviceName);
        RequestCAKeyPair requestCAKeyPair = builder.build();
        requestCAKeyPair.setAuthentication(new BasicAuthentication(DEFAULT_USER, DEFAULT_PASSWORD));
        return requestCAKeyPair;
    }

    private HttpResponse getMockHttpResponse(JSONConvertible responseData, boolean isEmptyResponse) throws UnsupportedEncodingException {
        HttpResponse httpResponse = mock(HttpResponse.class);
        if(isEmptyResponse){
            when(httpResponse.getEntity()).thenReturn(null);
        } else{
            when(httpResponse.getEntity()).thenReturn(new StringEntity(responseData.toJSON()));
        }
        return httpResponse;
    }

    private HttpResponse getMockInfoHttpResponse(ConnectionStatusCode type) throws UnsupportedEncodingException {
        HttpResponse httpResponse = mock(HttpResponse.class);
        StatusLine statusLine = mock(StatusLine.class);
        if(type.equals(ConnectionStatusCode.UNAUTHORIZED_ERROR)){
            when(httpResponse.getStatusLine()).thenReturn(statusLine);
            when(statusLine.getStatusCode()).thenReturn(401);
        } else if(type.equals(ConnectionStatusCode.CONNECTION_ERROR)){
            when(httpResponse.getStatusLine()).thenReturn(statusLine);
            when(statusLine.getStatusCode()).thenReturn(402);
        } else if(type.equals(ConnectionStatusCode.SUCCESS)){
            when(httpResponse.getEntity()).thenReturn(new StringEntity(INFO_OPERATIONS.toJSON()));
            when(httpResponse.getStatusLine()).thenReturn(statusLine);
            when(statusLine.getStatusCode()).thenReturn(200);
        }
        return httpResponse;
    }

    private CertificateAuthorityServiceImpl prepareRevokeRequestMockService(RequestCARevoke revokeRequest, boolean isEmptyResponse) throws HttpAuthenticationException, IOException, CertificateAuthorityServiceException {
        CertificateAuthorityServiceImpl certificateAuthorityServiceSpy = spy(new CertificateAuthorityServiceImpl());
        RevokeResponseData mockResponseData = new RevokeResponseData("success", "");
        HttpResponse mockHttpResponse = getMockHttpResponse(mockResponseData, isEmptyResponse);
        doReturn(mockHttpResponse).when(certificateAuthorityServiceSpy).getHttpResponse(BASE_URL, revokeRequest);
        return certificateAuthorityServiceSpy;
    }

}
